
/*
  $Id: setproctitle.c,v 1.3 2004/08/08 22:35:42 ldv Exp $

  Copyright (C) 2004  Dmitry V. Levin <ldv@altlinux.org>

  The setproctitle library interface.

  This file is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This file is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
*/

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>

#include "setproctitle.h"

extern char *__progname, *__progname_full;

extern char **environ;

static char *title_buffer = 0;
static size_t title_buffer_size = 0;
static char *title_progname, *title_progname_full;

static int saved_argc;
static char **saved_argv;

int
setproctitle (const char *fmt, ...)
{
	if (!title_buffer || !title_buffer_size)
	{
		errno = ENOMEM;
		return -1;
	}

	if (saved_argc && saved_argv)
		memset (saved_argv, '\0',
			saved_argc * sizeof (saved_argv[0]));

	ssize_t written;

	if (fmt)
	{
		ssize_t written2;
		va_list ap;

		written =
			snprintf (title_buffer, title_buffer_size, "%s: ",
				  title_progname);
		if (written < 0 || written >= title_buffer_size)
			return -1;

		va_start (ap, fmt);
		written2 =
			vsnprintf (title_buffer + written,
				   title_buffer_size - written, fmt, ap);
		va_end (ap);
		if (written2 < 0 || written2 >= title_buffer_size - written)
			return -1;
	} else
	{
		written =
			snprintf (title_buffer, title_buffer_size, "%s",
				  title_progname);
		if (written < 0 || written >= title_buffer_size)
			return -1;
	}

	written = strlen (title_buffer);
	memset (title_buffer + written, '\0', title_buffer_size - written);

	return 0;
}

int
_init (int argc, char *argv[], char *envp[])
{
	char   *begin_of_buffer = 0, *end_of_buffer = 0;
	int     i;

	for (i = 0; i < argc; ++i)
	{
		if (!begin_of_buffer)
			begin_of_buffer = argv[i];
		if (!end_of_buffer || end_of_buffer + 1 == argv[i])
			end_of_buffer = argv[i] + strlen (argv[i]);
	}

	for (i = 0; envp[i]; ++i)
	{
		if (!begin_of_buffer)
			begin_of_buffer = envp[i];
		if (!end_of_buffer || end_of_buffer + 1 == envp[i])
			end_of_buffer = envp[i] + strlen (envp[i]);
	}

	if (!end_of_buffer)
		return 0;

	char  **new_environ = malloc ((i + 1) * sizeof (envp[0]));

	if (!new_environ)
		return 0;

	for (i = 0; envp[i]; ++i)
	{
		if (!(new_environ[i] = strdup (envp[i])))
		{
			for (--i; i >= 0; --i)
				free (new_environ[i]);
			free (new_environ);
			return 0;
		}
	}
	new_environ[i] = 0;

	if (__progname_full)
	{
		title_progname_full = strdup (__progname_full);

		if (!title_progname_full)
		{
			for (--i; i >= 0; --i)
				free (new_environ[i]);
			free (new_environ);
			return 0;
		}

		char   *p = strrchr (title_progname_full, '/');

		if (p)
			title_progname = p + 1;
		else
			title_progname = title_progname_full;

		__progname_full = title_progname_full;
		__progname = title_progname;
	}

	environ = new_environ;

	saved_argc = argc;
	saved_argv = argv;

	title_buffer = begin_of_buffer;
	title_buffer_size = end_of_buffer - begin_of_buffer;

	return 0;
}
